@@ -41,41 +41,36 @@ int Kill(char *dev, int force)
 	 * Definitely not safe.
 	 */
 
-	mdp_super_t super;
+	void *super;
 	int fd, rv = 0;
+	struct supertype *st;
 		
-	fd = open(dev, O_RDWR);
+	fd = open(dev, O_RDWR|O_EXCL);
 	if (fd < 0) {
 		fprintf(stderr, Name ": Couldn't open %s for write - not zeroing\n",
 			dev);
 		return 1;
-	} 
-	rv = load_super(fd, &super);
-	if (force && rv >= 5)
-		rv = 0; /* ignore bad data in superblock */
-	switch(rv) {
-	case 1:
-		fprintf(stderr, Name ": cannot file device size for %s: %s\n",
-			dev, strerror(errno));
-		break;
-	case 2:
-		fprintf(stderr, Name ": %s is too small for md.\n", dev);
-		break;
-	case 3:
-	case 4:
-		fprintf(stderr, Name ": cannot access superblock on %s.\n", dev);
-		break;
-	case 5:
-	case 6:
-		fprintf(stderr, Name ": %s does not appear to have an MD superblock.\n", dev);
-		break;
 	}
-	if (!rv) {
-		memset(&super, 0, sizeof(super));
-		if (store_super(fd, &super)) {
+	st = guess_super(fd);
+	if (st == NULL) {
+		fprintf(stderr, Name ": Unrecognised md component device - %s\n", dev);
+		return 1;
+	}
+	rv = st->ss->load_super(st, fd, &super, dev);
+	if (force && rv >= 2)
+		rv = 0; /* ignore bad data in superblock */
+	if (rv== 0 || (force && rv >= 2)) {
+		mdu_array_info_t info;
+		info.major_version = -1; /* zero superblock */
+		free(super);
+		st->ss->init_super(st, &super, &info, "");
+		if (st->ss->store_super(st, fd, super)) {
 			fprintf(stderr, Name ": Could not zero superblock on %s\n",
 				dev);
 			rv = 1;
+		} else if (rv) {
+			fprintf(stderr, Name ": superblock zeroed anyway\n");
+			rv = 0;
 		}
 	}
 	close(fd);
